#ifndef DEFS_DEFS_H
#define DEFS_DEFS_H

//useful macros

//class generating macros:
//adds:
//typename super
//unsigned int getClassSize(void)
//const char *getClassName(void)

#define BEGIN_CHILDCLASS(classname,parentclassname) \
class classname : public parentclassname { \
public: \
	typedef parentclassname super; \
	unsigned int getClassSize(void) { return sizeof(classname); }	\
	virtual const char *getClassName(void) { return #classname; }	\
	virtual const char *getSuperClassName(void) { return #parentclassname; }

#define BEGIN_BASECLASS(classname)	\
class classname {	\
public:	\
	typedef void super;	\
	unsigned int getClassSize(void) { return sizeof(classname); }	\
	virtual const char *getClassName(void) { return #classname; }	\
	virtual const char *getSuperClassName(void) { return "void"; }

#define ENDCLASS()	};


//random number [0,1)
#define frand()	((float)rand()/(float)RAND_MAX)

//random number [-1,1)
#define crand()	(frand()*2.f-1.f)

//repeat a definition for return type and for object const type being const
//'head' contains the function return type, name, and parameters
//'body' contains the code (without surrounding brackets -- they were so repetative i just added them to the macro)
#define REPEAT_WITH_CONST(head, body)	\
	head { body; }				\
	const head const { body; }


/**
 * returns the number of elements in an array
 */
#ifndef numberof
#define numberof(x)	(sizeof((x)) / sizeof((x)[0]))
#endif

/**
 * returns the element just after the last element in an array
 * useful for cycling pointers over arrays in for() loops
 */
#ifndef endof
#define endof(x)	((x) + numberof((x)))
#endif

/**
 * returns the x'th bit set
 */
#define bitflag(x)	((1<<(x)))

//tests y's bounds to numberof(x) - useful for static arrays
#define safeget(x,y)		((y) >= 0 && (y) < numberof(x) ? (x)[y] : NULL)

//test's x for existence before dereferencing by 'y' - useful for pointers
#define saferef(x,y)		((x) ? (x)->y : NULL)

/**
 * returns the max of the two values
 */
#ifdef max	//lets make sure this macro is upper-case
#undef max	//I saw this trick somewhere else
#endif //max
#ifndef MAX
#define MAX(x,y)	(((x)>(y)?(x):(y)))
#endif //MAX

#ifdef min
#undef min
#endif
#ifndef MIN
#define MIN(x,y)	(((x)>(y)?(y):(x)))
#endif

//returns a nonneg modulo of a float val
inline float ifmod(float val, int mod) {
	if (val < 0) {
		val = -val;
		val -= (int)(val / mod) * mod;
		val = (float)mod - val;
	} else {
		val -= (int)(val / mod) * mod;
	}
	return val;
}

#define FLOOR_FTOI(x)		((x) < 0 ? (int)(x) - 1 : (int)(x))

//function access, so we dont have to re-evaluate 'x' twice
inline int floor_ftoi(float x) { return FLOOR_FTOI(x);}

//<vector> stuff

#define VECTOR_DELETE_ALL(v)	{ int i = (int)v.size()-1; for (; i >= 0; i--) if (v[i]) delete v[i]; v.resize(0); }

//in msvc 5.0, vectors wont compile without these two comparators:
#define VECTOR_FIX_MSVC50(x)	\
static bool operator< (const x &a, const x &b) { return !memcmp(&a, &b, sizeof(x)); }	\
static bool operator== (const x &a, const x &b) { return !memcmp(&a, &b, sizeof(x)); }
#define MSVC50_VECTOR_FIX	VECTOR_FIX_MSVC50

#endif //DEFS_DEFS_H
